@@ -7,17 +7,19 @@ Convert::BER::XS - I<very> low level BER en-/decoding
  use Convert::BER::XS ':all';
 
  my $ber = ber_decode $buf
-    or die "unable to decode SNMP v1/v2c Message";
+    or die "unable to decode SNMP message";
 
- # the above results in a data structure consisting of (class, tag,
- # constructed, data) tuples. here is such a message, SNMPv1 trap
- # with a cisoc mac change notification
+ # The above results in a data structure consisting of (class, tag,
+ # constructed, data) tuples. Below is such a message, SNMPv1 trap
+ # with a Cisco mac change notification.
+ # Did you know that Cisco is in the news almost every week because
+ # of some backdoor password or other extremely stupid security bug?
 
  [ ASN_UNIVERSAL, ASN_SEQUENCE, 1,
    [
       [ ASN_UNIVERSAL, ASN_INTEGER32, 0, 0 ], # snmp version 1
       [ ASN_UNIVERSAL, 4, 0, "public" ], # community
-      [ ASN_CONTEXT, 4, 1, # CHOICE, constructed
+      [ ASN_CONTEXT, 4, 1, # CHOICE, constructed - trap PDU
          [
             [ ASN_UNIVERSAL, ASN_OBJECT_IDENTIFIER, 0, "1.3.6.1.4.1.9.9.215.2" ], # enterprise oid
             [ ASN_APPLICATION, 0, 0, "\x0a\x00\x00\x01" ], # SNMP IpAddress, 10.0.0.1
@@ -64,6 +66,8 @@ Convert::BER::XS - I<very> low level BER en-/decoding
 
 =head1 DESCRIPTION
 
+WARNING: Before release 1.0, the API is not considered stable in any way.
+
 This module implements a I<very> low level BER/DER en-/decoder.
 
 If is tuned for low memory and high speed, while still maintaining some
@@ -106,6 +110,25 @@ array-reference):
 
    [CLASS, TAG, CONSTRUCTED, DATA]
 
+To avoid non-descriptive hardcoded array index numbers, this module
+defines symbolic constants to access these members: C<BER_CLASS>,
+C<BER_TAG>, C<BER_CONSTRUCTED> and C<BER_DATA>.
+
+Also, the first three members are integers with a little caveat: for
+performance reasons, these are readonly and shared, so you must not modify
+them (increment, assign to them etc.) in any way. You may modify the
+I<DATA> member, and you may re-assign the array itself, e.g.:
+
+   $ber = ber_decode $binbuf;
+
+   # the following is NOT legal:
+   $ber->[BER_CLASS] = ASN_PRIVATE; # ERROR, readonly(!)
+
+   # but all of the following are fine:
+   $ber->[BER_DATA] = "string";
+   $ber->[BER_DATA] = [ASN_UNIVERSAL, ASN_INTEGER32, 0, 123];
+   @$ber = (ASN_APPLICATION, SNMP_TIMETICKS, 1000);
+
 I<CLASS> is something like a namespace for I<TAG>s - there is the
 C<ASN_UNIVERSAL> namespace which defines tags common to all ASN.1
 implementations, the C<ASN_APPLICATION> namespace which defines tags for
@@ -145,6 +168,111 @@ string in place of some nice decoded value.
 
 See the SYNOPSIS for an example of such an encoded tuple representation.
 
+=head2 DECODING AND ENCODING
+
+=over
+
+=item $tuple = ber_decoded $bindata
+
+Decodes binary BER data in C<$bindata> and returns the resulting BER
+tuple. Croaks on any decoding error, so the returned C<$tuple> is always
+valid.
+
+=item $bindata = ber_encode $tuple
+
+Encodes the BER tuple into a BER/DER data structure.
+
+=back
+
+=head2 HELPER FUNCTIONS
+
+Working with a 4-tuple for every value can be annoying. Or, rather, I<is>
+annoying. To reduce this a bit, this module defines a number of helper
+functions, both to match BER tuples and to conmstruct BER tuples:
+
+=head3 MATCH HELPERS
+
+Thse functions accept a BER tuple as first argument and either paertially
+or fully match it. They often come in two forms, one which exactly matches
+a value, and one which only matches the type and returns the value.
+
+They do check whether valid tuples are passed in and croak otherwise. As
+a ease-of-use exception, they usually also accept C<undef> instead of a
+tuple reference. in which case they silently fail to match.
+
+=over
+
+=item $bool = ber_is $tuple, $class, $tag, $constructed, $data
+
+This takes a BER C<$tuple> and matches its elements agains the privded
+values, all of which are optional - values that are either missing or
+C<undef> will be ignored, the others will be matched exactly (e.g. as if
+you used C<==> or C<eq> (for C<$data>)).
+
+Some examples:
+
+   ber_is $tuple, ASN_UNIVERSAL, ASN_SEQUENCE, 1
+      orf die "tuple is not an ASN SEQUENCE";
+
+   ber_is $tuple, ASN_UNIVERSAL, ASN_NULL
+      or die "tuple is not an ASN NULL value";
+
+   ber_is $tuple, ASN_UNIVERSAL, ASN_INTEGER32, 0, 50
+      or die "BER integer must be 50";
+
+=item $seq = ber_is_seq $tuple
+
+Returns the sequence members (the array of subvalues) if the C<$tuple> is
+an ASN SEQUENCE, i.e. the C<BER_DATA> member. If the C<$tuple> is not a
+sequence it returns C<undef>. For example, SNMP version 1/2c/3 packets all
+consist of an outer SEQUENCE value:
+
+   my $ber = ber_decode $snmp_data;
+
+   my $snmp = ber_is_seq $ber
+      or die "SNMP packet invalid: does not start with SEQUENCE";
+
+   # now we know $snmp is a sequence, so decode the SNMP version
+
+   my $version = ber_is_i32 $snmp->[0]
+      or die "SNMP packet invalid: does not start with version number";
+
+=item $bool = ber_is_i32 $tuple, $i32
+
+Returns a true value if the C<$tuple> represents an ASN INTEGER32 with
+the value C<$i32>.
+
+=item $i32 = ber_is_i32 $tuple
+
+Returns true (and extracts the integer value) if the C<$tuple> is an ASN
+INTEGER32. For C<0>, this function returns a special value that is 0 but
+true.
+
+=item $bool = ber_is_oid $tuple, $oid_string
+
+Returns true if the C<$tuple> represents an ASN_OBJECT_IDENTIFIER
+that exactly matches C$oid_string>. Exmaple:
+
+   ber_is_oid $tuple, "1.3.6.1.4"
+      or die "oid must be 1.3.6.1.4";
+
+=item $oid = ber_is_oid $tuple
+
+Returns true (and extracts the OID string) if the C<$tuple> is an ASN
+OBJECT IDENTIFIER. Otherwise, it returns C<undef>.
+
+=back
+
+=head3 CONSTRUCTION HELPERS
+
+=over
+
+=item $tuple = ber_i32 $value
+
+Constructs a new C<ASN_INTEGER32> tuple.
+
+=back
+
 =head2 RELATIONSHIP TO L<Convert::BER> and L<Convert::ASN1>
 
 This module is I<not> the XS version of L<Convert::BER>, but a different
@@ -161,7 +289,7 @@ use common::sense;
 use XSLoader ();
 use Exporter qw(import);
 
-our $VERSION = 0.1;
+our $VERSION = 0.2;
 
 XSLoader::load __PACKAGE__, $VERSION;
 
